import logging
import html
from telegram import Update
from telegram.ext import ContextTypes
from .base import InfoCommandsBase
from src.config.config import Config
from src.utils.token_display_formatter import get_formatter
from datetime import datetime

logger = logging.getLogger(__name__)

class RiskCommands(InfoCommandsBase):
    """Handles all risk management-related commands."""

    def __init__(self, trading_engine, notification_manager):
        super().__init__(trading_engine, notification_manager)
        self.formatter = get_formatter()

    async def risk_command(self, update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
        """Handle the /risk command to show key portfolio risk metrics."""
        try:
            # Check for unused arguments and notify user
            if context.args:
                await self._reply(update, "Note: The `/risk` command shows portfolio-level data. Token-specific arguments are ignored.")

            # Fetch necessary data
            risk_metrics = self.trading_engine.stats.get_risk_metrics()
            positions = self.trading_engine.get_positions()
            balance_data = self.trading_engine.get_balance()

            if not risk_metrics:
                await self._reply(update, "Risk metrics are not available yet.")
                return

            # Calculate total unrealized P&L
            total_unrealized_pnl = sum(float(p.get('unrealizedPnl', 0.0)) for p in positions)
            portfolio_value = float(balance_data.get('total', {}).get('USDC', 0.0))
            
            # Calculate portfolio risk percentage
            portfolio_risk_pct = (total_unrealized_pnl / portfolio_value * 100) if portfolio_value > 0 else 0.0

            # Format the message
            message = [
                "<b>🛡️ Portfolio Risk Analysis</b>\n",
                f"Here are the key risk metrics for your portfolio:\n"
            ]

            # Max Drawdown
            max_drawdown_pct = risk_metrics.get('max_drawdown_percentage', 0.0)
            drawdown_start_date_iso = risk_metrics.get('drawdown_start_date')
            drawdown_date_str = ""
            if drawdown_start_date_iso:
                try:
                    drawdown_date = datetime.fromisoformat(drawdown_start_date_iso).strftime('%Y-%m-%d')
                    drawdown_date_str = f" (since {drawdown_date})"
                except (ValueError, TypeError):
                    logger.warning(f"Could not parse drawdown_start_date: {drawdown_start_date_iso}")
            
            message.append(f"• <b>Max Drawdown:</b> {max_drawdown_pct:.2f}%{drawdown_date_str}")
            
            # Sharpe Ratio
            sharpe_ratio = risk_metrics.get('sharpe_ratio')
            if sharpe_ratio is not None:
                message.append(f"• <b>Sharpe Ratio:</b> {sharpe_ratio:.2f}")
            else:
                message.append("• <b>Sharpe Ratio:</b> N/A (Insufficient data)")

            # Overall Portfolio Risk
            pnl_emoji = "🟢" if total_unrealized_pnl >= 0 else "🔴"
            message.append(f"• <b>Open P&L Risk:</b> {pnl_emoji} {portfolio_risk_pct:.2f}% ({await self.formatter.format_price_with_symbol(total_unrealized_pnl)})")

            message.append("\n<i>This provides a high-level overview of portfolio risk. For position-specific details, use /positions.</i>")

            await self._reply(update, "\n".join(message))
            
        except Exception as e:
            logger.error(f"Error in risk command: {e}", exc_info=True)
            await self._reply(update, "❌ Error getting risk information. Please try again later.")
